# coding=utf-8
from django import test
from django.shortcuts import resolve_url
from django.test import RequestFactory, TestCase, override_settings
from django.test.utils import TestContextDecorator
from django_powerbank.testing.base import AssertionsMx
from mock import Mock, patch

from django_opt_out import factories, models, signals, views
from django_opt_out.utils import get_opt_out_url


class CaptureSignal(TestContextDecorator):
    """
    A function decorator to connect/disconnect mock handler functions to signals
    """

    def __init__(self, signal, kwarg_name='handler', handler=None):
        super(CaptureSignal, self).__init__(kwarg_name=kwarg_name)
        self.signal = signal
        self.handler = handler or Mock()

    def enable(self):
        self.signal.connect(self.handler)
        return self.handler

    def disable(self):
        self.signal.connect(self.handler)


class OptOutConfirmTests(TestCase):
    @override_settings(OPT_OUT_GOODBYE_VIEW='mocked_goodbye')
    def test_overriden_success_url(self):
        view = views.OptOutConfirm()
        view.object = factories.OptOutFactory(pk=1, secret='7ebc5d464a6485e4b64f', email='ymoore@hotmail.com')
        url = view.get_success_url()
        self.assertEqual('/mocked_goodbye/1/7ebc5d464a6485e4b64f/ymoore@hotmail.com/', url)

    def test_success_url(self):
        view = views.OptOutConfirm()
        view.object = factories.OptOutFactory(pk=1, secret='7ebc5d464a6485e4b64f', email='ymoore@hotmail.com')
        url = view.get_success_url()
        self.assertEqual('/opt-out/success/1/7ebc5d464a6485e4b64f/ymoore@hotmail.com', url)


class OptOutConfirmGetTests(TestCase, AssertionsMx):
    @CaptureSignal(signals.opt_out_visited)
    def test_empty(self, handler):
        url = resolve_url("django_opt_out:OptOutConfirm")
        response = test.Client().get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual({}, response.context_data['form'].errors)
        self.assertTrue(handler.called)

    def test_simple(self):
        url = get_opt_out_url("foo@bar.com")
        response = test.Client().get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual({}, response.context_data['form'].errors)
        self.assertEqual("foo@bar.com", response.context_data['form']['email'].initial)

    def test_with_feedback(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(10)
        url = get_opt_out_url("foo@bar.com")
        response = test.Client().get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual({}, response.context_data['form'].errors)
        choices = [pk for pk, label in response.context_data['form'].fields['feedback'].choices]
        self.assertEqual(set((f.pk for f in feedback)), set(choices))

    def test_only_default_feedback(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(3)
        tag1 = factories.OptOutTagFactory()
        factories.OptOutFeedbackFactory.create_batch(3, tags=(tag1,))
        url = get_opt_out_url("foo@bar.com")
        response = test.Client().get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual({}, response.context_data['form'].errors)
        choices = [pk for pk, label in response.context_data['form'].fields['feedback'].choices]
        self.assertEqual(set((f.pk for f in feedback)), set(choices))

    def test_feedback_based_on_tags(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(3)
        tag1 = factories.OptOutTagFactory()
        feedback.extend(factories.OptOutFeedbackFactory.create_batch(3, tags=(tag1,)))
        tag2 = factories.OptOutTagFactory()
        factories.OptOutFeedbackFactory.create_batch(3, tags=(tag2,))
        tag3 = factories.OptOutTagFactory()
        feedback.extend(factories.OptOutFeedbackFactory.create_batch(3, tags=(tag3,)))
        url = get_opt_out_url("foo@bar.com", tag1.name, tag3.name)
        response = test.Client().get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual({}, response.context_data['form'].errors)
        choices = [pk for pk, label in response.context_data['form'].fields['feedback'].choices]
        self.assertEqual(set((f.pk for f in feedback)), set(choices))

    def test_feedback_checked_by_default(self):
        factories.OptOutFeedbackFactory.create_batch(3)
        feedback = factories.OptOutFeedbackFactory.create_batch(3, default=True)
        url = get_opt_out_url("foo@bar.com")
        response = test.Client().get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual({}, response.context_data['form'].errors)
        initial = response.context_data['form'].fields['feedback'].initial
        self.assertEqual([f.pk for f in feedback], initial)

    def test_default_feedback(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(3, default=True)
        request = RequestFactory().get('/', data={'tag': 'default'})
        view = views.OptOutConfirm(request=request)
        form = view.get_form()
        items = list(form.fields['feedback'].queryset)
        self.assertEqual(set(feedback), set(items))

    def test_default_feedback_on(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(3, default=True)
        request = RequestFactory().get('/', data={'tag': 'default:on'})
        view = views.OptOutConfirm(request=request)
        form = view.get_form()
        items = list(form.fields['feedback'].queryset)
        self.assertEqual(set(feedback), set(items))

    def test_default_feedback_true(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(3, default=True)
        request = RequestFactory().get('/', data={'tag': 'default:true'})
        view = views.OptOutConfirm(request=request)
        form = view.get_form()
        items = list(form.fields['feedback'].queryset)
        self.assertEqual(set(feedback), set(items))

    def test_default_feedback_True(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(3, default=True)
        request = RequestFactory().get('/', data={'tag': 'default:True'})
        view = views.OptOutConfirm(request=request)
        form = view.get_form()
        items = list(form.fields['feedback'].queryset)
        self.assertEqual(set(feedback), set(items))

    def test_default_feedback_1(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(3, default=True)
        request = RequestFactory().get('/', data={'tag': 'default:1'})
        view = views.OptOutConfirm(request=request)
        form = view.get_form()
        items = list(form.fields['feedback'].queryset)
        self.assertEqual(set(feedback), set(items))

    def test_default_feedback_0(self):
        factories.OptOutFeedbackFactory.create_batch(3, default=True)
        request = RequestFactory().get('/', data={'tag': 'default:0'})
        view = views.OptOutConfirm(request=request)
        form = view.get_form()
        self.assertEqual(0, form.fields['feedback'].queryset.count())


@patch('django_opt_out.plugins.sparkpost.hooks.client', Mock())
class OptOutConfirmPostTests(TestCase, AssertionsMx):
    @CaptureSignal(signals.opt_out_submitted)
    def test_just_email(self, handler):
        url = resolve_url("django_opt_out:OptOutConfirm")
        response = test.Client().post(url, data={'email': 'foo@bar.com'})
        self.assertNoFormErrors(response)
        item = models.OptOut.objects.first()
        self.assertIsNotNone(item)
        self.assertEqual(item.email, 'foo@bar.com')
        url = resolve_url("django_opt_out:OptOutSuccess", item.pk, item.secret, item.email)
        self.assertRedirects(response, url)
        self.assertTrue(handler.called)

    def test_comment(self):
        url = resolve_url("django_opt_out:OptOutConfirm")
        response = test.Client().post(url, data={'email': 'foo@bar.com', 'comment': 'Dynda, dynda stryczka cień!'})
        self.assertNoFormErrors(response)
        item = models.OptOut.objects.first()
        self.assertIsNotNone(item)
        self.assertEqual(item.comment, 'Dynda, dynda stryczka cień!')

    def test_valid_tag(self):
        tag = factories.OptOutTagFactory(name='source')
        url = get_opt_out_url("foo@bar.com", "source")
        response = test.Client().post(url, data={'email': 'foo@bar.com'})
        self.assertNoFormErrors(response)
        item = models.OptOut.objects.first()
        item_tag = item.tags.all().first()
        self.assertEqual(tag, item_tag.tag)
        self.assertIsNone(item_tag.value)

    def test_valid_tag_value(self):
        factories.OptOutTagFactory(name='source')
        url = get_opt_out_url("foo@bar.com", "source:some")
        response = test.Client().post(url, data={'email': 'foo@bar.com'})
        self.assertNoFormErrors(response)
        item = models.OptOut.objects.first()
        self.assertEqual('some', item.tags.all().first().value)

    def test_multiple_tags(self):
        factories.OptOutTagFactory(name='source')
        factories.OptOutTagFactory(name='flag')
        url = get_opt_out_url("foo@bar.com", "source:some", "flag")
        response = test.Client().post(url, data={'email': 'foo@bar.com'})
        self.assertNoFormErrors(response)
        item = models.OptOut.objects.first()
        self.assertEqual(['source', 'flag'], list(item.tags.all().values_list('tag__name', flat=True)))

    def test_feedback(self):
        questions = factories.OptOutFeedbackFactory.create_batch(10)[3:7]
        url = get_opt_out_url("foo@bar.com")
        response = test.Client().post(url, data={'email': 'foo@bar.com', 'feedback': [q.pk for q in questions]})
        self.assertNoFormErrors(response)
        item = models.OptOut.objects.first()
        self.assertEqual(set(questions), set(item.feedback.all()))

    def test_confirmed(self):
        url = get_opt_out_url("foo@bar.com")
        response = test.Client().post(url, data={'email': 'foo@bar.com'})
        self.assertNoFormErrors(response)
        item = models.OptOut.objects.first()
        self.assertIsNotNone(item)
        self.assertIsNotNone(item.confirmed)

    @override_settings(OUT_OUT_REQUIRE_CONFIRMATION=False)
    def test_not_confirmed(self):
        url = get_opt_out_url("foo@bar.com")
        response = test.Client().post(url, data={'email': 'foo@bar.com'})
        self.assertNoFormErrors(response)
        item = models.OptOut.objects.first()
        self.assertIsNotNone(item)
        self.assertIsNone(item.confirmed)


class OptOutUpdateGetTests(TestCase, AssertionsMx):
    def test_404(self):
        url = resolve_url("django_opt_out:OptOutUpdate", 2, 'a', 'foo@bar')
        response = test.Client().get(url)
        self.assertEqual(404, response.status_code)

    def test_forbidden_bad_secret(self):
        item = factories.OptOutFactory()
        url = resolve_url("django_opt_out:OptOutUpdate", item.pk, item.secret[:-1], item.email)
        response = test.Client().get(url)
        self.assertEqual(403, response.status_code)

    def test_forbidden_bad_email(self):
        item = factories.OptOutFactory()
        url = resolve_url("django_opt_out:OptOutUpdate", item.pk, item.secret, 'a' + item.email)
        response = test.Client().get(url)
        self.assertEqual(403, response.status_code)

    def test_authorized(self):
        item = factories.OptOutFactory()
        url = resolve_url("django_opt_out:OptOutUpdate", item.pk, item.secret, item.email)
        response = test.Client().get(url)
        self.assertEqual(200, response.status_code)
        self.assertNoFormErrors(response)

    def test_feedback_based_on_saved_tags(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(3)
        tag1 = factories.OptOutTagFactory()
        feedback.extend(factories.OptOutFeedbackFactory.create_batch(3, tags=(tag1,)))
        tag2 = factories.OptOutTagFactory()
        factories.OptOutFeedbackFactory.create_batch(3, tags=(tag2,))
        item = factories.OptOutFactory()
        item.tags.create(tag=tag1)
        url = resolve_url("django_opt_out:OptOutUpdate", item.pk, item.secret, item.email)
        response = test.Client().get(url)
        self.assertEqual(200, response.status_code)
        self.assertNoFormErrors(response)
        choices = [pk for pk, label in response.context_data['form'].fields['feedback'].choices]
        self.assertEqual(set((f.pk for f in feedback)), set(choices))

    def test_initial_feedback(self):
        feedback = factories.OptOutFeedbackFactory.create_batch(7)[3:5]
        item = factories.OptOutFactory()
        item.feedback.add(*feedback)
        url = resolve_url("django_opt_out:OptOutUpdate", item.pk, item.secret, item.email)
        response = test.Client().get(url)
        self.assertEqual(200, response.status_code)
        self.assertNoFormErrors(response)
        choices = set(response.context_data['form'].fields['feedback'].initial)
        self.assertEqual(set((f.pk for f in feedback)), set(choices))


class OptOutUpdatePostTests(TestCase, AssertionsMx):
    def test_save(self):
        item = factories.OptOutFactory()
        url = resolve_url("django_opt_out:OptOutUpdate", item.pk, item.secret, item.email)
        response = test.Client().post(url, data={'email': item.email})
        self.assertNoFormErrors(response)
        self.assertRedirects(response, resolve_url("django_opt_out:OptOutSuccess", item.pk, item.secret, item.email))


@patch('django_opt_out.plugins.sparkpost.hooks.client', Mock())
class OptOutSuccessTests(TestCase, AssertionsMx):
    @CaptureSignal(signals.opt_out_deleted)
    def test_post(self, handler):
        item = factories.OptOutFactory()
        url = resolve_url("django_opt_out:OptOutSuccess", item.pk, item.secret, item.email)
        response = test.Client().post(url)
        self.assertNoFormErrors(response)
        self.assertRedirects(response, resolve_url("django_opt_out:OptOutRemoved"))
        self.assertTrue(handler.called)



# -*- coding: utf-8 -*-
from django.conf.urls import url

from . import views as v

app_name = 'django_opt_out'

urlpatterns = [
    url(r'^$', v.OptOutConfirm.as_view(), name='OptOutConfirm'),
    url(r'^success/(?P<pk>[\d]+)/(?P<secret>[\w]+)/(?P<email>[^/]+)$', v.OptOutSuccess.as_view(), name='OptOutSuccess'),
    url(r'^update/(?P<pk>[\d]+)/(?P<secret>[\w]+)/(?P<email>[^/]+)$', v.OptOutUpdate.as_view(), name='OptOutUpdate'),
    url(r'^removed$', v.OptOutRemoved.as_view(), name='OptOutRemoved'),
]



# -*- coding: utf-8 -*-
from django.conf import global_settings
from django.db import models
from django.utils import translation
from django.utils.translation import ugettext_lazy as _
from django_powerbank.db.models.base import BaseModel
from django_powerbank.db.models.fields import AutoSlugField, JSONField, SecretField

from . import signals


class OptOutTag(BaseModel):
    name = models.CharField(verbose_name=_('tag name'), max_length=30)
    description = models.CharField(verbose_name=_('description'), max_length=250, default='')

    class Meta:
        default_related_name = "tag_names"
        verbose_name = _('tag')
        verbose_name_plural = _('tags')

    def __str__(self):
        return self.name


class OptOutFeedback(BaseModel):
    text = models.CharField(verbose_name=_('text'), max_length=250)
    slug = AutoSlugField(source_field='text', keep_existing=True, max_length=30, unique=True)
    default = models.BooleanField(verbose_name=_('checked by default'), default=False)
    ordinal = models.PositiveIntegerField(verbose_name=_('ordinal'), default=0)
    tags = models.ManyToManyField(OptOutTag, verbose_name=_('tags'), blank=True)

    class Meta:
        default_related_name = "feedback"
        verbose_name = _('feedback option')
        verbose_name_plural = _('feedback options')

    def __str__(self):
        return self.text

    def trans(self, language=None):
        if language is None:
            language = translation.get_language()
        label = OptOutFeedbackTranslation.objects.filter(feedback=self, language=language).first() or self
        return label.text


class OptOutFeedbackTranslation(BaseModel):
    feedback = models.ForeignKey(OptOutFeedback, on_delete=models.CASCADE)
    language = models.CharField(max_length=5, choices=global_settings.LANGUAGES)
    text = models.CharField(verbose_name=_('question'), max_length=250)

    class Meta:
        default_related_name = 'translations'
        unique_together = (('feedback', 'text'), ('feedback', 'language'))
        verbose_name = _('feedback option translation')
        verbose_name_plural = _('feedback options translations')


class OptOut(BaseModel):
    email = models.EmailField(verbose_name=_('email'))
    ts = models.DateTimeField(verbose_name=_('update timestamp'), auto_now=True)
    confirmed = models.DateTimeField(verbose_name=_('confirmation timestamp'), null=True, blank=True)
    data = JSONField(verbose_name=_('extra data'), null=True, blank=True)
    comment = models.TextField(verbose_name=_('comment'), null=True, blank=True)
    feedback = models.ManyToManyField(OptOutFeedback, verbose_name=_('feedback'))
    secret = SecretField(source_field="email", max_length=200)

    # Signature elements
    ssl = models.NullBooleanField()
    host = models.CharField(max_length=200, null=True, blank=True)
    ip = models.GenericIPAddressField(null=True, blank=True)
    ua = models.CharField(max_length=200, null=True, blank=True)
    cookies = models.TextField(null=True, blank=True)

    class Meta:
        default_related_name = "out_outs"
        verbose_name = _('opt out')
        verbose_name_plural = _('oup outs')

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        super().save(force_insert, force_update, using, update_fields)
        signals.opt_out_saved.send_robust(self.__class__, opt_out=self)


class OptOutTagValue(BaseModel):
    opt_out = models.ForeignKey(OptOut, on_delete=models.CASCADE)
    tag = models.ForeignKey(OptOutTag, on_delete=models.PROTECT)
    value = models.CharField(verbose_name=_('tag value'), max_length=80, db_index=True, null=True, blank=True)

    class Meta:
        default_related_name = "tags"
        verbose_name = _('tag')
        verbose_name_plural = _('tags')

    def __str__(self):
        return "OptOut-{}:{}:{}".format(self.opt_out_id, self.tag.name, self.value)



# coding=utf-8
import logging
from pathlib import Path

import tablib
from django.core.exceptions import ValidationError
from django.core.management.base import BaseCommand
from django.utils.translation import ugettext_lazy as _

from ... import models, resources

log = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Imports default opt-out feedback options to empty database'

    data_files = (
        ("tags.csv", resources.OptOutTagResource),
        ("feedback.csv", resources.OptOutFeedbackResource),
        ("feedback-translations.csv", resources.OptOutFeedbackTranslationResource),
    )

    def add_arguments(self, parser):
        parser.add_argument('--force', action='store_true', dest='force', default=False, help='Overwrite existing data')
        parser.add_argument('--on-empty', action='store_true', dest='on_empty', default=False, help='Only if database is emtpy')

    def handle(self, *args, **options):
        if not options['force']:
            try:
                assert models.OptOutTag.objects.count() == 0
                assert models.OptOutFeedback.objects.count() == 0
                assert models.OptOutFeedbackTranslation.objects.count() == 0
            except AssertionError as ex:
                if options['on_empty']:
                    log.info("Database is not empty. Ignoring import.")
                    return
                raise ex
        self.import_all()

    def import_all(self):
        for name, resource in self.data_files:
            self.import_data_file(name, resource)

    @staticmethod
    def import_data_file(name, resource):
        data_file = Path(__file__).parent / name
        log.debug("Looking data file: %s", str(data_file))
        with (data_file).open(encoding='UTF-8') as data:
            dataset = tablib.Dataset().load(data.read(), format='csv')

        if dataset.width < 1:
            raise ValidationError(_("Data set has no columns"))
        if dataset.height < 1:
            raise ValidationError(_("Data set has no rows"))

        # log.debug("Trying dryrun...")
        # OrganizationLocationResource().import_data(dataset, dry_run=True, raise_errors=True)
        log.debug(_('Trying to import {}').format(name))
        result = resource().import_data(dataset, dry_run=False, raise_errors=True)
        log.info(_('Imported {} rows from {}').format(result.total_rows, name))



# -*- coding: utf-8
from __future__ import absolute_import

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class DjangoOptOutSparkPostConfig(AppConfig):
    name = 'django_opt_out.plugins.sparkpost'
    verbose_name = _('Messaging Opt-Outs SparkPost plugin')

    def ready(self):
        from . import hooks  # noqa F401



# coding=utf-8
import factory
import faker
from django.contrib.auth import get_user_model
from django.utils import timezone

from . import models

fake = faker.Faker()


class UserFactory(factory.DjangoModelFactory):
    first_name = factory.Faker('first_name')
    last_name = factory.Faker('last_name')
    email = factory.Faker('email')
    username = factory.Sequence(lambda n: fake.user_name() + str(n))
    is_staff = False
    is_active = True

    class Meta:
        model = get_user_model()


class OptOutTagFactory(factory.DjangoModelFactory):
    name = factory.Faker('domain_word')

    class Meta:
        model = models.OptOutTag


class OptOutFactory(factory.DjangoModelFactory):
    email = factory.Faker('email')
    ts = factory.LazyFunction(timezone.now)

    class Meta:
        model = models.OptOut


class OptOutTagValueFactory(factory.DjangoModelFactory):
    opt_out = factory.SubFactory(OptOutTagFactory)
    tag = factory.SubFactory(OptOutTagFactory)

    class Meta:
        model = models.OptOutTagValue


class OptOutFeedbackFactory(factory.DjangoModelFactory):
    text = factory.Faker('sentence')

    class Meta:
        model = models.OptOutFeedback

    @factory.post_generation
    def tags(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of groups were passed in, use them
            self.tags.add(*extracted)


class OptOutFeedbackTranslationFactory(factory.DjangoModelFactory):
    feedback = factory.SubFactory(OptOutFeedbackFactory)
    language = 'pl'
    text = factory.Faker('sentence')

    class Meta:
        model = models.OptOutFeedbackTranslation



# coding=utf-8
from django.test import TestCase

from django_opt_out import admin, factories, models


class OptOutFeedbackAdminTests(TestCase):
    def test_all_tag_names(self):
        item = factories.OptOutFeedbackFactory()
        item.tags.create(name="a")
        item.tags.create(name="b")
        item.tags.create(name="c")
        self.assertEqual("a, b, c", admin.OptOutFeedbackAdmin(models.OptOutFeedback, None).all_tag_names(item))



#!/usr/bin/env python
import logging
import os
import sys

logging.basicConfig(format='%(asctime)s %(levelname)-7s %(thread)-5d %(filename)s:%(lineno)s | %(funcName)s | %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
logging.getLogger().setLevel(logging.DEBUG)
logging.getLogger("PIL.Image").setLevel(logging.WARNING)
logging.disable(logging.NOTSET)
logging.getLogger('environ').setLevel(logging.INFO)

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "website.settings")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



# coding=utf-8
from import_export import resources, widgets

from . import models


class OptOutTagResource(resources.ModelResource):
    class Meta:
        model = models.OptOutTag
        export_order = ('name',)
        fields = (
            'id',
            'name',
        )


class OptOutFeedbackResource(resources.ModelResource):
    tags = resources.Field(attribute='tags', column_name='tags', widget=widgets.ManyToManyWidget(models.OptOutTag, field='name'))

    class Meta:
        model = models.OptOutFeedback
        export_order = ('text',)
        fields = (
            'id',
            'text',
            'slug',
            'default',
            'ordinal',
            'tags',
        )


class OptOutFeedbackTranslationResource(resources.ModelResource):
    feedback = resources.Field(attribute='feedback', column_name='feedback', widget=widgets.ForeignKeyWidget(models.OptOutFeedback, field='slug'))

    class Meta:
        model = models.OptOutFeedbackTranslation
        export_order = ('text',)
        fields = (
            'id',
            'feedback',
            'text',
            'language',
        )


class OptOutResource(resources.ModelResource):
    feedback = resources.Field(attribute='tags', column_name='tags', widget=widgets.ManyToManyWidget(models.OptOutFeedback, field='slug'))

    class Meta:
        model = models.OptOut
        export_order = ('ts',)
        fields = (
            'email',
            'ts',
            'confirmed',
            'data',
            'comment',
            'feedback',
            'secret',
            'ssl',
            'ip',
            'ua',
            'cookies',
        )


class OptOutTagValueResource(resources.ModelResource):
    opt_out = resources.Field(attribute='tags', column_name='tags', widget=widgets.ForeignKeyWidget(models.OptOut))
    tag = resources.Field(attribute='tags', column_name='tags', widget=widgets.ForeignKeyWidget(models.OptOutTag, 'name'))

    class Meta:
        model = models.OptOutTagValue
        export_order = ('text',)
        fields = (
            'opt_out',
            'tag',
            'value',
        )



# coding=utf-8
import logging

import django.dispatch

log = logging.getLogger(__name__)

opt_out_visited = django.dispatch.Signal(providing_args=["view", "request", "context"])
opt_out_submitted = django.dispatch.Signal(providing_args=["view", "request", "opt_out"])
opt_out_deleted = django.dispatch.Signal(providing_args=["view", "request", "opt_out"])

opt_out_saved = django.dispatch.Signal(providing_args=["opt_out"])


def send_signal(signal, *args, **kwargs):
    """Log eny errors returned in the signal call responses"""
    results = signal.send_robust(*args, **kwargs)
    for receiver, response in results:
        if isinstance(response, Exception):
            log.error(response, exc_info=(type(response), response, response.__traceback__))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import os
import re
import uuid
from glob import glob
from os.path import basename, splitext

from pip.req import parse_requirements

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup
install_requires = parse_requirements(
    os.path.join(os.path.dirname(__file__), "requirements.txt"),
    session=uuid.uuid1()
)


def get_version(*file_paths):
    """Retrieves the version from path"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    print("Looking for version in: {}".format(filename))
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


version = get_version("src", "website", "__init__.py")

setup(
    name='django-opt-out-example',
    version=version,
    description="""Example app for django-opt-out""",
    long_description="""This is an example project that should not be published or installed""",
    author="Janusz Skonieczny",
    author_email='js+pypi@bravelabs.pl',
    url='https://github.com/wooyek/django-opt-out',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    install_requires=[str(r.req) for r in install_requires] + ['Django>=1.10'],
    license="MIT license",
    zip_safe=False,
    keywords='django-opt-out-example',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='runtests.run_tests',
    tests_require=['pytest', 'pytest-django'],
    # https://docs.pytest.org/en/latest/goodpractices.html#integrating-with-setuptools-python-setup-py-test-pytest-runner
    setup_requires=['pytest-runner'],
)



# coding=utf-8
from django.test import TestCase
from django.utils import translation

from django_opt_out import factories, forms


class OptOutFormTests(TestCase):

    def test_translated_feedback(self):
        feedback = factories.OptOutFeedbackFactory()
        text = 'Zażółć gęślą jaźń'
        feedback.translations.create(text=text, language='pl')
        self.assertEqual('en-us', translation.get_language())
        with translation.override('pl'):
            form = forms.OptOutForm()
            self.assertEqual([(1, text)], list(form.fields['feedback'].choices))



# coding=utf-8
from django.contrib import admin
from import_export.admin import ImportExportMixin

from . import models, resources


@admin.register(models.OptOut)
class OptOutAdmin(ImportExportMixin, admin.ModelAdmin):
    resource_class = resources.OptOutResource
    list_display = ('ts', 'email', 'ip', 'host', 'ua')
    list_filter = ('ip', 'host', 'ua', 'feedback', 'tags')
    readonly_fields = ('email', 'ts', 'comment', 'ip', 'host', 'ua', 'cookies', 'data', 'ssl')
    date_hierarchy = 'ts'


@admin.register(models.OptOutTag)
class OptOutTagAdmin(ImportExportMixin, admin.ModelAdmin):
    resource_class = resources.OptOutTagResource
    list_display = ('name', 'description')


@admin.register(models.OptOutFeedbackTranslation)
class OptOutFeedbackTranslationAdmin(ImportExportMixin, admin.ModelAdmin):
    resource_class = resources.OptOutFeedbackTranslationResource
    list_display = ('text', 'language', 'feedback')


class OptOutFeedbackTranslationInline(admin.TabularInline):
    resource_class = resources.OptOutFeedbackResource
    model = models.OptOutFeedbackTranslation


@admin.register(models.OptOutFeedback)
class OptOutFeedbackAdmin(ImportExportMixin, admin.ModelAdmin):
    resource_class = resources.OptOutFeedbackResource
    list_display = ('text', 'slug', 'default', 'ordinal', 'all_tag_names')
    list_filter = ('tags', 'default')
    inlines = [OptOutFeedbackTranslationInline]

    def all_tag_names(self, obj):
        return ", ".join((t.name for t in obj.tags.all()))

    all_tag_names.short_description = 'Tag names'

    # noinspection PyUnusedLocal,PyMethodMayBeStatic
    def queryset(self, request, queryset):
        return queryset.prefech_related('tags')



# -*- coding: utf-8
from __future__ import absolute_import, unicode_literals

from django.conf.urls import url
from django.views.generic import TemplateView

from website.urls import urlpatterns

urlpatterns += [
    url(r'^mocked_goodbye/(?P<pk>[\d]+)/(?P<secret>[\w]+)/(?P<email>[^/]+)/', TemplateView.as_view(template_name='nonexistent'), name='mocked_goodbye'),
]



# -*- coding: utf-8 -*-
from django.conf.urls import url

from . import views as v

app_name = 'django_opt_out_sparkpost'

urlpatterns = [
    url(r'^spark-post/unsubscribe-hook$', v.SparkPostUnsubscribeWebhook.as_view(), name='SparkPostUnsubscribeWebhook'),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import os
import re
import sys
import uuid
from glob import glob
from os.path import basename, splitext

from pip.req import parse_requirements

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

install_requires = parse_requirements(
    os.path.join(os.path.dirname(__file__), "requirements", "production.txt"),
    session=uuid.uuid1()
)

test_requirements = parse_requirements(
    os.path.join(os.path.dirname(__file__), "requirements", "testing.txt"),
    session=uuid.uuid1()
)


def get_version(*file_paths):
    """Retrieves the version from path"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    print("Looking for version in: {}".format(filename))
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


version = get_version("src", "django_opt_out", "__init__.py")

if sys.argv[-1] == 'publish':
    try:
        import wheel

        print("Wheel version: ", wheel.__version__)
    except ImportError:
        print('Wheel library missing. Please run "pip install wheel"')
        sys.exit()
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'bump':
    print("Bumping version number:")
    os.system("bumpversion patch --no-tag")
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on git:")
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

setup(
    name='django-opt-out',
    version=version,
    description="""Allow everybody to unsubscribe your messages, user accounts are not required.""",
    long_description=readme + '\n\n' + history,
    author="Janusz Skonieczny",
    author_email='js+pypi@bravelabs.pl',
    url='https://github.com/wooyek/django-opt-out',
    packages=find_packages('src', exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/!(test*).py')],
    entry_points={
        'console_scripts': [
            'django_opt_out=django_opt_out.cli:main'
        ]
    },
    include_package_data=True,
    exclude_package_data={
        '': ['test*.py', 'tests/*.env', '**/tests.py'],
    },
    # python_requires='>=3.4',
    install_requires=[str(r.req) for r in install_requires] + ['Django>=1.10'],
    extras_require={
        'factories': ['factory-boy'],
        'sparkpost': ['sparkpost'],
    },
    license="MIT license",
    zip_safe=False,
    keywords='django-opt-out',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='runtests.run_tests',
    tests_require=[str(r.req) for r in test_requirements],
    # https://docs.pytest.org/en/latest/goodpractices.html#integrating-with-setuptools-python-setup-py-test-pytest-runner
    setup_requires=['pytest-runner'],
)



# -*- coding: utf-8
"""
Django settings for example project.

Generated by wooyek/cookiecutter-django-app

For more information on this file, see
https://docs.djangoproject.com/en/dev/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/dev/ref/settings/

# Before go-live check if your settings are suitable for production
# See https://docs.djangoproject.com/en/1.9/howto/deployment/checklist/
"""

from __future__ import absolute_import, unicode_literals

import logging
import os
import socket
from pathlib import Path

import environ

logging.basicConfig(format='%(asctime)s %(levelname)-7s %(thread)-5d %(filename)s:%(lineno)s | %(funcName)s | %(message)s', datefmt='%H:%M:%S')
logging.getLogger().setLevel(logging.DEBUG)
logging.disable(logging.NOTSET)

logging.debug("Settings loading: %s" % __file__)

# Project root folder
BASE_DIR = Path(__file__).parents[3]

env = environ.Env()

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = env('DEBUG', bool, False)
logging.debug("DEBUG: %s", DEBUG)
ENVIRONMENT_NAME = env('ENVIRONMENT_NAME', default='')

SITE_ID = 1
BASE_URL = "http://dev.example.com:5555"

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = env("SECRET_KEY")

INTERNAL_IPS = [
    '127.0.0.1',
]

ALLOWED_HOSTS = [
    '127.0.0.1',
    'localhost',
    '.example.com',
    'vagrant',
]

BASE_DOMAIN = HOSTNAME = socket.gethostname().lower()
if 'ALLOWED_HOSTS' in os.environ and os.environ['ALLOWED_HOSTS'].strip():
    hosts = os.environ['ALLOWED_HOSTS'].split(" ")
    BASE_HOST = hosts[0]
    BASE_URL = "https://" + BASE_HOST
    ALLOWED_HOSTS = [host.strip() for host in hosts if host.strip()]

SECURE_SSL_REDIRECT = env.bool('SECURE_SSL_REDIRECT', default=False)

ADMINS = (
    ("Administrator", "{domain} admin <admin@{domain}>".format(domain=BASE_DOMAIN)),
)
if 'ADMINS' in os.environ:
    from email.utils import getaddresses

    admins = os.environ['ALLOWED_HOSTS'].split(";")
    addreses = getaddresses(admins)
    ADMINS = [(name, named_email) for ((name, email), named_email) in zip(addreses, admins)]

DEFAULT_FROM_EMAIL = env('DEFAULT_FROM_EMAIL', default="Help <help@{domain}>".format(domain=BASE_DOMAIN))
HELP_EMAIL = env('HELP_EMAIL', default=DEFAULT_FROM_EMAIL)
ERR_EMAIL = env('ERR_EMAIL', default="errors@{domain}".format(domain=BASE_DOMAIN))
SERVER_EMAIL = env('SERVER_EMAIL', default="Errors <errors@{domain}>".format(domain=BASE_DOMAIN))
EMAIL_SUBJECT_PREFIX = env('EMAIL_SUBJECT_PREFIX', default='[OPT-OUT] ')

# Database
# https://docs.djangoproject.com/en/dev/ref/settings/#databases
DATABASES = {}

if "DATABASE_URL" in os.environ:  # pragma: no cover
    DATABASES['default'] = env.db()
    DATABASES['default']['TEST'] = {'NAME': env("DATABASE_TEST_NAME", default=None)}
    DATABASES['default']['OPTIONS'] = {
        'options': '-c search_path=gis,public,pg_catalog',
        'sslmode': 'require',
    }
else:
    DATABASES['default'] = {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": str(BASE_DIR / 'data' / 'db.sqlite3'),
        'TEST': {
            "NAME": ":memory:",
        }
    }

logging.debug("DATABASES: %s", DATABASES['default']['NAME'])

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'bootstrapform',
    'import_export',
    'django_opt_out.apps.DjangoOptOutConfig',
    'django_opt_out.plugins.sparkpost.apps.DjangoOptOutSparkPostConfig',
]

MIDDLEWARE_CLASSES = [
    'django.middleware.security.SecurityMiddleware',
    'whitenoise.middleware.WhiteNoiseMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'website.urls'

TEMPLATES = [
    {
        # https://docs.djangoproject.com/en/dev/topics/templates/#django.template.backends.django.DjangoTemplates
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [str(BASE_DIR / 'templates'), ],
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
            ],
        },
    },
]

WSGI_APPLICATION = 'website.wsgi.application'

# The email backend to use. For possible shortcuts see django.core.mail.
# The default is to use the SMTP backend.
# Third-party backends can be specified by providing a Python path
# to a module that defines an EmailBackend class.
EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'

# https://github.com/joke2k/django-environ#email-settings
if 'EMAIL_URL' in os.environ and os.environ['EMAIL_URL'].strip():
    EMAIL_CONFIG = env.email_url('EMAIL_URL')
    vars().update(EMAIL_CONFIG)

# Internationalization
# https://docs.djangoproject.com/en/dev/topics/i18n/

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True

# https://docs.djangoproject.com/en/1.9/topics/i18n/translation/#how-django-discovers-language-preference
import django_powerbank  # noqa F402 isort:skip
import django_opt_out  # noqa F402 isort:skip

LOCALE_PATHS = [
    str(Path(django_opt_out.__file__).parent / 'locales'),
    str(BASE_DIR / 'locales'),
    str(Path(django_powerbank.__file__).parent / 'locales'),
]

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/dev/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = str(BASE_DIR / 'static')

# https://docs.djangoproject.com/en/1.9/ref/settings/#logging
LOGGING = {
    'version': 1,
    # Setting this to True will disable for eg. preexisting Celery loggers
    'disable_existing_loggers': False,
    'formatters': {
        'short': {
            'format': '%(asctime)s %(levelname)-7s %(thread)-5d %(message)s',
            'datefmt': '%H:%M:%S',
        },
        # this may slow down the app a little, due to
        'verbose': {
            'format': '%(asctime)s %(levelname)-7s %(thread)-5d %(name)s %(filename)s:%(lineno)s | %(funcName)s | %(message)s',
            'datefmt': '%Y-%m-%d %H:%M:%S',
        },
        'heroku': {
            'format': '%(levelname)-7s %(thread)-5d %(name)s %(filename)s:%(lineno)s | %(funcName)s | %(message)s',
            'datefmt': '%Y-%m-%d %H:%M:%S',
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },

    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'short',
            'level': 'DEBUG',
        },
        'mail_admins': {
            'level': 'ERROR',
            'email_backend': 'django.core.mail.backends.smtp.EmailBackend',
            'class': 'django.utils.log.AdminEmailHandler',
        },
    },
    'loggers': {
        'django.template': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False,
        },
        'django': {
            'handlers': ['console'],
            'propagate': False,
            'level': 'INFO',
        },
        'django.security.DisallowedHost': {
            'handlers': [],
            'propagate': False,
        },
        'suds': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': True,
        },
        'django.db.backends': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'factory.generate': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'factory.containers': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'dinja2': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'environ.environ': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'INFO',
        },
        'pil': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'pil.image': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'WARNING',
        },
        'django_opt_out': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'DEBUG',
        },
        # 'django.security.DisallowedHost': {
        #     'handlers': [],
        #     'propagate': False,
        # },
    },
    'root': {
        'level': 'DEBUG',
        'handlers': ['console'],
    }
}

if env('MAIL_ADMINS_ON_ERROR', bool, default=True):
    # This duplicates Sentry functionality
    LOGGING['loggers']['django.request'] = {
        'handlers': ['mail_admins'],
        'level': 'ERROR',
        'propagate': True,
    }

SPARKPOST_API_KEY = env('SPARKPOST_API_KEY', default=None)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-10 23:12
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django_powerbank.db.models.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='OptOut',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254, verbose_name='email')),
                ('ts', models.DateTimeField(auto_now=True, verbose_name='update timestamp')),
                ('confirmed', models.DateTimeField(blank=True, null=True, verbose_name='confirmation timestamp')),
                ('data', django_powerbank.db.models.fields.JSONField(blank=True, default='{}', null=True, verbose_name='extra data')),
                ('comment', models.TextField(blank=True, null=True, verbose_name='comment')),
                ('secret', django_powerbank.db.models.fields.SecretField(max_length=200)),
                ('ssl', models.NullBooleanField()),
                ('host', models.CharField(blank=True, max_length=200, null=True)),
                ('ip', models.GenericIPAddressField(blank=True, null=True)),
                ('ua', models.CharField(blank=True, max_length=200, null=True)),
                ('cookies', models.TextField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'opt out',
                'default_related_name': 'out_outs',
                'verbose_name_plural': 'oup outs',
            },
        ),
        migrations.CreateModel(
            name='OptOutFeedback',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.CharField(max_length=250, verbose_name='text')),
                ('slug', django_powerbank.db.models.fields.AutoSlugField(max_length=30, unique=True)),
                ('default', models.BooleanField(default=False, verbose_name='checked by default')),
                ('ordinal', models.PositiveIntegerField(default=0, verbose_name='ordinal')),
            ],
            options={
                'verbose_name': 'feedback option',
                'default_related_name': 'feedback',
                'verbose_name_plural': 'feedback options',
            },
        ),
        migrations.CreateModel(
            name='OptOutFeedbackTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language', models.CharField(choices=[('af', 'Afrikaans'), ('ar', 'Arabic'), ('ast', 'Asturian'), ('az', 'Azerbaijani'), ('bg', 'Bulgarian'), ('be', 'Belarusian'), ('bn', 'Bengali'), ('br', 'Breton'), ('bs', 'Bosnian'), ('ca', 'Catalan'), ('cs', 'Czech'), ('cy', 'Welsh'), ('da', 'Danish'), ('de', 'German'), ('dsb', 'Lower Sorbian'), ('el', 'Greek'), ('en', 'English'), ('en-au', 'Australian English'), ('en-gb', 'British English'), ('eo', 'Esperanto'), ('es', 'Spanish'), ('es-ar', 'Argentinian Spanish'), ('es-co', 'Colombian Spanish'), ('es-mx', 'Mexican Spanish'), ('es-ni', 'Nicaraguan Spanish'), ('es-ve', 'Venezuelan Spanish'), ('et', 'Estonian'), ('eu', 'Basque'), ('fa', 'Persian'), ('fi', 'Finnish'), ('fr', 'French'), ('fy', 'Frisian'), ('ga', 'Irish'), ('gd', 'Scottish Gaelic'), ('gl', 'Galician'), ('he', 'Hebrew'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hsb', 'Upper Sorbian'), ('hu', 'Hungarian'), ('ia', 'Interlingua'), ('id', 'Indonesian'), ('io', 'Ido'), ('is', 'Icelandic'), ('it', 'Italian'), ('ja', 'Japanese'), ('ka', 'Georgian'), ('kk', 'Kazakh'), ('km', 'Khmer'), ('kn', 'Kannada'), ('ko', 'Korean'), ('lb', 'Luxembourgish'), ('lt', 'Lithuanian'), ('lv', 'Latvian'), ('mk', 'Macedonian'), ('ml', 'Malayalam'), ('mn', 'Mongolian'), ('mr', 'Marathi'), ('my', 'Burmese'), ('nb', 'Norwegian Bokmål'), ('ne', 'Nepali'), ('nl', 'Dutch'), ('nn', 'Norwegian Nynorsk'), ('os', 'Ossetic'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Brazilian Portuguese'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('sq', 'Albanian'), ('sr', 'Serbian'), ('sr-latn', 'Serbian Latin'), ('sv', 'Swedish'), ('sw', 'Swahili'), ('ta', 'Tamil'), ('te', 'Telugu'), ('th', 'Thai'), ('tr', 'Turkish'), ('tt', 'Tatar'), ('udm', 'Udmurt'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('vi', 'Vietnamese'), ('zh-hans', 'Simplified Chinese'), ('zh-hant', 'Traditional Chinese')], max_length=5)),
                ('text', models.CharField(max_length=250, verbose_name='question')),
                ('feedback', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='django_opt_out.OptOutFeedback')),
            ],
            options={
                'verbose_name': 'feedback option translation',
                'default_related_name': 'translations',
                'verbose_name_plural': 'feedback options translations',
            },
        ),
        migrations.CreateModel(
            name='OptOutTag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30, verbose_name='tag name')),
                ('description', models.CharField(default='', max_length=250, verbose_name='description')),
            ],
            options={
                'verbose_name': 'tag',
                'default_related_name': 'tag_names',
                'verbose_name_plural': 'tags',
            },
        ),
        migrations.CreateModel(
            name='OptOutTagValue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(blank=True, db_index=True, max_length=80, null=True, verbose_name='tag value')),
                ('opt_out', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tags', to='django_opt_out.OptOut')),
                ('tag', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='tags', to='django_opt_out.OptOutTag')),
            ],
            options={
                'verbose_name': 'tag',
                'default_related_name': 'tags',
                'verbose_name_plural': 'tags',
            },
        ),
        migrations.AddField(
            model_name='optoutfeedback',
            name='tags',
            field=models.ManyToManyField(blank=True, related_name='feedback', to='django_opt_out.OptOutTag', verbose_name='tags'),
        ),
        migrations.AddField(
            model_name='optout',
            name='feedback',
            field=models.ManyToManyField(related_name='out_outs', to='django_opt_out.OptOutFeedback', verbose_name='feedback'),
        ),
        migrations.AlterUniqueTogether(
            name='optoutfeedbacktranslation',
            unique_together=set([('feedback', 'language'), ('feedback', 'text')]),
        ),
    ]



# coding=utf-8
from django import test
from django.shortcuts import resolve_url
from django.test import TestCase
from django.test.utils import override_settings
from mock import patch

from .test_views import CaptureSignal
from django_opt_out.models import OptOut
from django_opt_out.plugins.sparkpost import signals


class SparkPostHookTests(TestCase):
    def test_opt_out_created(self):
        self.assertEqual(0, OptOut.objects.all().count())
        url = resolve_url('django_opt_out_sparkpost:SparkPostUnsubscribeWebhook')
        test.Client().post(url, data=list_unsubscribe, content_type="application/json")
        opt_out = OptOut.objects.all().first()
        self.assertEqual('recipient@example.com', opt_out.email)
        self.assertIsNotNone(opt_out.data)

    @CaptureSignal(signals.list_unsubscribe)
    def test_list_unsubscribe(self, handler):
        url = resolve_url('django_opt_out_sparkpost:SparkPostUnsubscribeWebhook')
        test.Client().post(url, data=list_unsubscribe, content_type="application/json")
        self.assertTrue(handler.called)
        args, kwargs = handler.call_args
        self.assertEqual('recipient@example.com', kwargs['email'])

    @CaptureSignal(signals.link_unsubscribe)
    def test_link_unsubscribe(self, handler):
        url = resolve_url('django_opt_out_sparkpost:SparkPostUnsubscribeWebhook')
        test.Client().post(url, data=link_unsubscribe, content_type="application/json")
        self.assertTrue(handler.called)
        args, kwargs = handler.call_args
        self.assertEqual('recipient@example.com', kwargs['email'])

    @CaptureSignal(signals.list_unsubscribe, 'list_handler')
    @CaptureSignal(signals.link_unsubscribe, 'link_handler')
    def test_multi(self, link_handler, list_handler):
        url = resolve_url('django_opt_out_sparkpost:SparkPostUnsubscribeWebhook')
        test.Client().post(url, data=unsubscribe_multiple, content_type="application/json")

        self.assertTrue(link_handler.called)
        args, kwargs = link_handler.call_args
        self.assertEqual('recipient@example.com', kwargs['email'])

        self.assertTrue(list_handler.called)
        args, kwargs = list_handler.call_args
        self.assertEqual('recipient@example.com', kwargs['email'])

    @override_settings(SPARKPOST_API_KEY='not-valid')
    @patch('django_opt_out.plugins.sparkpost.hooks.client.suppression_list.create')
    def test_confirm_creates_suppression(self, create):
        url = resolve_url("django_opt_out:OptOutConfirm")
        test.Client().post(url, data={'email': 'foo@bar.com'})
        self.assertTrue(create.called)

list_unsubscribe = """[{"msys":{"unsubscribe_event":{"type":"list_unsubscribe","campaign_id":"Example Campaign Name","customer_id":"1","delv_method":"esmtp","event_id":"92356927693813856","friendly_from":"sender@example.com","ip_address":"127.0.0.1","ip_pool":"Example-Ip-Pool","mailfrom":"recipient@example.com","message_id":"000443ee14578172be22","msg_from":"sender@example.com","msg_size":"1337","num_retries":"2","queue_time":"12","rcpt_meta":{"customKey":"customValue"},"rcpt_tags":["male","US"],"rcpt_to":"recipient@example.com","raw_rcpt_to":"recipient@example.com","rcpt_type":"cc","routing_domain":"example.com","sending_ip":"127.0.0.1","subaccount_id":"101","subject":"Summer deals are here!","template_id":"templ-1234","template_version":"1","timestamp":"1454442600","transmission_id":"65832150921904138"}}}]"""  # noqa E501
link_unsubscribe = """[{"msys":{"unsubscribe_event":{"type":"link_unsubscribe","campaign_id":"Example Campaign Name","customer_id":"1","delv_method":"esmtp","event_id":"92356927693813856","friendly_from":"sender@example.com","ip_address":"127.0.0.1","ip_pool":"Example-Ip-Pool","mailfrom":"recipient@example.com","message_id":"000443ee14578172be22","msg_from":"sender@example.com","msg_size":"1337","num_retries":"2","queue_time":"12","rcpt_meta":{"customKey":"customValue"},"rcpt_tags":["male","US"],"rcpt_to":"recipient@example.com","raw_rcpt_to":"recipient@example.com","rcpt_type":"cc","routing_domain":"example.com","sending_ip":"127.0.0.1","subaccount_id":"101","subject":"Summer deals are here!","template_id":"templ-1234","template_version":"1","timestamp":"1454442600","transmission_id":"65832150921904138","user_agent":"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2272.118 Safari/537.36"}}}]"""  # noqa E501
unsubscribe_multiple = """
[
  {
    "msys": {
      "unsubscribe_event": {
        "type": "list_unsubscribe",
        "campaign_id": "Example Campaign Name",
        "customer_id": "1",
        "delv_method": "esmtp",
        "event_id": "92356927693813856",
        "friendly_from": "sender@example.com",
        "ip_address": "127.0.0.1",
        "ip_pool": "Example-Ip-Pool",
        "mailfrom": "recipient@example.com",
        "message_id": "000443ee14578172be22",
        "msg_from": "sender@example.com",
        "msg_size": "1337",
        "num_retries": "2",
        "queue_time": "12",
        "rcpt_meta": {
          "customKey": "customValue"
        },
        "rcpt_tags": [
          "male",
          "US"
        ],
        "rcpt_to": "recipient@example.com",
        "raw_rcpt_to": "recipient@example.com",
        "rcpt_type": "cc",
        "routing_domain": "example.com",
        "sending_ip": "127.0.0.1",
        "subaccount_id": "101",
        "subject": "Summer deals are here!",
        "template_id": "templ-1234",
        "template_version": "1",
        "timestamp": "1454442600",
        "transmission_id": "65832150921904138"
      }
    }
  },
  {
    "msys": {
      "unsubscribe_event": {
        "type": "link_unsubscribe",
        "campaign_id": "Example Campaign Name",
        "customer_id": "1",
        "delv_method": "esmtp",
        "event_id": "92356927693813856",
        "friendly_from": "sender@example.com",
        "ip_address": "127.0.0.1",
        "ip_pool": "Example-Ip-Pool",
        "mailfrom": "recipient@example.com",
        "message_id": "000443ee14578172be22",
        "msg_from": "sender@example.com",
        "msg_size": "1337",
        "num_retries": "2",
        "queue_time": "12",
        "rcpt_meta": {
          "customKey": "customValue"
        },
        "rcpt_tags": [
          "male",
          "US"
        ],
        "rcpt_to": "recipient@example.com",
        "raw_rcpt_to": "recipient@example.com",
        "rcpt_type": "cc",
        "routing_domain": "example.com",
        "sending_ip": "127.0.0.1",
        "subaccount_id": "101",
        "subject": "Summer deals are here!",
        "template_id": "templ-1234",
        "template_version": "1",
        "timestamp": "1454442600",
        "transmission_id": "65832150921904138",
        "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2272.118 Safari/537.36"
      }
    }
  }
]
"""
